/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2007 by Walther Neuper
 * created Jul 18, 2007 4:06:22 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.gui;

import isac.browserdialog.IBrowserDialog;
import isac.gui.browser.BrowserFrame;
import isac.gui.browser.BrowserFrameSwing;
import isac.gui.browser.BrowserFrameRMI;
import isac.interfaces.IToGUI;
import isac.interfaces.IToUser;
import isac.session.IObjectManager;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIActionOnBrowserDialog;
import isac.useractions.UIActionOnWSDialog;
import isac.useractions.UserActionOnLink;
import isac.users.IUserManager;
import isac.util.CalcChanged;
import isac.util.CalcEvent;
import isac.util.ObjectManagerPaths;
import isac.wsdialog.DialogProtocolException;
import isac.wsdialog.IWorksheetDialog;

import java.io.Serializable;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import junit.framework.TestCase;

import org.apache.log4j.Logger;

/**
 * WindowApplication with code copied from original, just omitting Swing-stuff;
 * but these method-calls are to coarse-grained in comparison to
 * java-test/isac.gui.TestWindowApplicationMini and
 * isac.gui.TestWindowApplicationStub. Thus this Testcase has been stopped at an
 * intermediate state.
 * 
 * Still creates BrowserFrame instead MockBrowserFrame.
 */
public class TestWindowApplication extends TestCase implements Serializable,
        IToGUI, IToUser {

    private static final long serialVersionUID = 5939308541271990081L;

    private static final Logger logger = Logger
            .getLogger(TestWindowApplication.class.getName());

    private IBrowserDialog current_session_;

    IUserManager user_manager_;

    private int num_browser_frames_;

    private BrowserFrame example_browser_;

    private BrowserFrame theory_browser_;

    private BrowserFrame problem_browser_;

    private BrowserFrame method_browser_;

    public void testMain() {
        try { // because TestWindowApplication has already 1 'extends'
            UnicastRemoteObject.exportObject(this, 0);// 0 avoids _Stub error
        } catch (RemoteException e1) {
            e1.printStackTrace();
        }
        logger.info("-----TestWA: start test, login----------------------");
        this.login("x", "x");

        // this should be done in the 'MetaWA' together with checking user
        UserActionOnLink start_exp = new UserActionOnLink(
                EUIElement.UI_MINIBROWSER_LINK,
                EUIContext.UI_CONTEXT_MINIBROWSER,
                "file://"
                        + ObjectManagerPaths.SESSION_MANAGER_USER_PATH
                        + "exp/"
                        + "exp_IsacCore_Tests_1b.html#COMMAND_EXEC_EXAMPLE#exp_IsacCore_Tests_1b.xml");

        logger.info("-----TestWA: notifyBrowserDialog(start_exp)---------");
        // ### example_browser_.notifyBrowserDialog(start_exp);
        // WN070723: what goes wrong here (Swing-code in Worksheet{Swing} ?
        // see logger: <NEXT><Auto>-code seems to break in MockWorksheet.java)

        logger.info("-----TestWA: end testMain---------------------------");
    }

    /**
     * login method called from <code>LoginScreen</code> Initialize RMI
     * connection to <code>ObjectManager</code>.
     * 
     * an exact copy from isac.gui.WindowApplication TOOODO: get is out of WA
     * into 'MetaWA': WA is created on succes only
     */
    private void login(String username, String password) {
        String name = // WindowApplicationPaths.OBJECT_MANAGER_RMI;
        "//localhost/isac-ObjectManager";
        try {
            Object lookup = Naming.lookup(name);
            user_manager_ = ((IObjectManager) lookup).getUserManager();
        } catch (RemoteException exc) {
            System.err.println("no rmiregistry runing on server " + name
                    + " no connection possible --- try later");
            System.err.println("serverCom exception: " + exc.getMessage());
            exc.printStackTrace();
        } catch (NotBoundException exc) {
            System.err.println("servercom: notboundexc: " + exc.getMessage());
            System.err.println("server not found - try to connect to Proxy");
        } catch (java.net.MalformedURLException exc) {
            System.err.println("servercom: MalformedURLException: "
                    + exc.getMessage());
            exc.printStackTrace();
        }
        if (logger.isInfoEnabled())
            logger.info("WA->UM: TestWA#login --------------");
        try {
            current_session_ = user_manager_.login(username, password);
            current_session_.registerBrowserFrame((IToGUI) this);
            if (logger.isDebugEnabled())
                logger
                        .debug("TestWA#login: current_session_ registerBrowserFrame done");
        } catch (RemoteException e) {
        }
        if (current_session_ != null) {
            // loginScreen.dispose();
            // main_frame_.setVisible(true);
        } else {
            // loginScreen.showErrorMsg();
        }
    }

    /**
     * creates a new browser frame, inserts it to the internal frames and
     * registeres it to the browser dialog.
     * 
     * called by ?BrowserDialog?
     */
    private void openNewBrowserFrame(IBrowserDialog browser_dialog,
            String frame_name) {
        try {
            BrowserFrame browser_frame = new BrowserFrame(browser_dialog,
                    frame_name);
            // addInternalFrame(browser_frame);

            num_browser_frames_++;
            // NEW: + ATTENTION see order in
            // isac.session.initialiseWindowApplication!
            switch (num_browser_frames_) {
            case 1:
                example_browser_ = browser_frame;
                break;
            case 2:
                theory_browser_ = browser_frame;
                break;
            case 3:
                problem_browser_ = browser_frame;
                break;
            case 4:
                method_browser_ = browser_frame;
                break;
            }

            // browser_frame.setSize( IGuiConstants.BROWSER_MIN_SIZE_X,
            // IGuiConstants.BROWSER_MIN_SIZE_Y );
            // browser_frame.setLocation( IGuiConstants.BROWSER_POS_X +
            // (num_browser_frames_ * 20),
            // IGuiConstants.BROWSER_POS_Y + ( num_browser_frames_ * 20 ) );

            BrowserFrameRMI browser_framain_frame_rmi = new BrowserFrameRMI(
                    browser_frame);

            browser_dialog
                    .registerBrowserFrame((IToGUI) browser_framain_frame_rmi);

        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }

    // -----IToGUI-----------------------------------------------------------
    /**
     * @deprecated
     */
    public boolean doUIAction(IUserAction action) throws RemoteException {
        // TODO Auto-generated method stub
        return false;
    }

    /**
     * only has to handle UI_OPEN_WORKSHEET + UI_OPEN_BROWSER_FRAME ! exact copy
     * from isac.gui.WindowApplication#doUIAction(IUIAction action)
     */
    public void doUIAction(IUIAction action) throws RemoteException {

        if (logger.isInfoEnabled())
            logger
                    .info("WA<-DG: TestWA#doUIAction(IUIAction action) : Action is "
                            + action.getAction()
                            + ", Context is "
                            + action.getContext());

        // if it is not our context go back
        if (action.getContext() != EUIContext.UI_CONTEXT_WINDOW_APPLICATION)
            return;

        switch (action.getAction()) {
        case UI_OPEN_WORKSHEET: {
            UIActionOnWSDialog uaowd = (UIActionOnWSDialog) action;
            openNewWorksheet(uaowd.getWorksheetDialog());
            break;
        }
        case UI_OPEN_BROWSER_FRAME:
            UIActionOnBrowserDialog uaobd = (UIActionOnBrowserDialog) action;
            openNewBrowserFrame(uaobd.getBrowserDialog(), uaobd.getFrameName());
            break;
        }
    }

    /**
     * Open a new Worksheet and start the modeling and specification phase.
     * 
     * @param fmz
     *            Formalization (can be empty see UC 2.5.4) that the math engine
     *            needs to start a calculation
     * @param source
     *            Source of the Formalization (e.g. ExampleHierarchy or from
     *            Scratch)
     * @param type
     *            Requested ViewType for the CalcHead
     * @throws RemoteException
     */
    private void openNewWorksheet(IWorksheetDialog dialog)
            throws RemoteException {

        // InternalFrame, holding the Frames for the Worksheet and the
        // corresponding CalcHeadPanel
        // JInternalFrame internalFrame = createInternalFrame("Worksheet", true,
        // true, false, false);
        // internalFrame.setResizable(true);
        // internalFrame.setLayout(new BorderLayout());

        // DesktopPane, added to internalFrame to be able to hae multiple
        // InternalFrames in this InternalFrame
        // final JDesktopPane desktop = new JDesktopPane();
        // desktop.setSize(WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_X,
        // WORKSHEET_DESKTOP_PANE_INITIAL_SIZE_Y);

        // desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);

        // internalFrame.setContentPane(desktop); // , BorderLayout.CENTER);

        // main_frame_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        MockWorksheet ws = new MockWorksheet((IToWorksheetUser) this, dialog,
                null);
        // #############################################################################################################################################
        // ..INSTEAD Worksheet ws = new Worksheet(this, dialog, desktop);
        // new InternalFrame for the Worksheet
        // JInternalFrame ws_frame = new JInternalFrame();
        // ((BasicInternalFrameUI) ws_frame.getUI()).setNorthPane(null);
        // ws_frame.setSize(WORKSHEET_FRAME_INITIAL_SIZE_X, main_frame_
        // .getHeight()
        // - WORKSHEET_FRAME_INITIAL_OFFSET_Y);

        // Container c;
        // c = ws_frame.getContentPane();
        // c.setLayout(new BorderLayout());
        // c.add(ws, BorderLayout.CENTER);

        // NC060726: The Worksheet desktop manager must not have the
        // BorderLayout
        // desktop.setLayout(new BorderLayout());
        // desktop.add(ws_frame, BorderLayout.CENTER);
        // desktop.add(ws_frame);

        // try {
        // ws_frame.setMaximum(true);
        // } catch (PropertyVetoException e) {
        // e.printStackTrace();
        // }

        // ws_frame.setFocusable(false);
        // ws_frame.setVisible(true);

        // internalFrame.addInternalFrameListener(ws);

        // int width, height = 0;
        // height = internalFrame.getPreferredSize().height;
        // width = internalFrame.getPreferredSize().width;
        // make Frame a little smaller than the window
        // height = main_frame_.getHeight() - WORKSHEET_FRAME_INITIAL_OFFSET_Y;
        // width = width < 600 ? WORKSHEET_FRAME_INITIAL_SIZE_X : width;

        // JInternalFrame[] frames = worksheetDesktopPane.getAllFrames();
        // int num_frames = frames.length - 4;

        // this.addInternalFrame(internalFrame);

        // internalFrame.setLocation(num_frames * 20, num_frames * 20);
        // internalFrame.setPreferredSize(new Dimension(width, height));
        // internalFrame.pack();
        // internalFrame.setVisible(true);

        // main_frame_.setCursor(Cursor.getDefaultCursor());
    }

    public void addUIElement(IUIAction action) throws RemoteException {
        // TODO Auto-generated method stub

    }

    public boolean removeUIElement(IUIAction action) throws RemoteException {
        // TODO Auto-generated method stub
        return false;
    }

    public boolean isUIRunning() throws RemoteException {
        // TODO Auto-generated method stub
        return false;
    }

    // -----IToUser----------------------------------------------------------
    public void calcResponse(CalcEvent event) throws RemoteException {
        // TODO Auto-generated method stub

    }

}
