/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * * 
 * Use is subject to license terms.
 ******************************************************************/
package isac.gui;

import isac.gui.treetable.CalcModelHierarchy;
import isac.gui.treetable.CalcModelNode;
import isac.gui.treetable.CalculationModel;
import isac.gui.treetable.CustomTreeCellEditor;
import isac.gui.treetable.CustomTreeCellRenderer;
import isac.interfaces.ICalcElement;
import isac.interfaces.IGuiToGui;
import isac.interfaces.IToGUI;
import isac.interfaces.IToUser;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UIActionOnCalcElement;
import isac.useractions.UserAction;
import isac.useractions.UserActionOnCalcElement;
import isac.useractions.UserActionOnPosition;
import isac.util.CalcChanged;
import isac.util.Formalization;
import isac.util.formulae.Assumptions;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;
import isac.util.tactics.Tactic;
import isac.wsdialog.IWorksheetDialog;
import isac.wsdialog.DialogProtocolException;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyVetoException;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.tree.DefaultTreeCellEditor;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.apache.log4j.Logger;

/**
 * CR070723 This shall be Worksheet inheriting to WorksheetSwing and
 * WorksheetStub to be created; see java-tests/isac.gui.WorksheetSwing and
 * java-tests/isac.gui.WorksheetStub.
 * 
 * @see java-tests/isac.gui.WorksheetSwing
 * 
 * <code>Worksheet</code> is the component that is responsible for the
 * interaction with the user while doing a calculation. Furthermore it acts as a
 * visual represantiation of mathematical formulas in a calculation tree, that
 * is beeing executed by the isac knowledge interpreter.
 * 
 * The <code>Worksheet</code> is the view of the <code>WorksheetDialog</code>.
 * 
 * The data, that the <code>WorksheetDialog</code> delivers, is representated
 * in the hierarchical structure <code>JTree</code> or on the
 * <code>CalcHeadPanel</code>.
 * 
 * @author Mario Hochreiter 2003
 * @version 0.1 15/09/2003
 * @author Alois Kirchsteiger 2004
 * @author Manuel Koschuch 2005
 */
public class WorksheetStub extends JPanel implements Serializable {

    private static final long serialVersionUID = -3202174096489836501L;

    private static final Logger logger = Logger.getLogger(WorksheetStub.class
            .getName());

    /***************************************************************************
     * Mouse Listener for handling mouse events in the worksheet tree
     * 
     */
    // private MouseAdapter ml;
    /**
     * Context menu for each worksheet member
     */
    // private JPopupMenu popupMenu;
    /***************************************************************************
     * The last point where the user clicked. Needed to show the Context Menu.
     */
    // private Point mouse_point_;
    /***************************************************************************
     * Used for close listener and to add the Next and Auto buttons
     */
    // private IToWorksheetUser windowApplication_;
    /***************************************************************************
     * The common actions that are applied to all nodes of the worksheet are
     * kept in this popu menu handler. The node specific commands are kept in
     * the handler of the node self.
     * 
     */
    // private WorksheetPopupHandler commonActionsOnCalcElements_;
    // ****HashMaps for UIActions on CalcElments
    // HashMap containing the actions with the menu_items as key
    // private HashMap<JComponent, IUIAction> dynamic_actions_by_menu_items_;
    // *************Contanier for dynamic buttons -- CONTEXT_CALCHEAD
    // private Vector<Integer> dynamic_action_ids_with_context_calchead_;
    protected IToUser worksheet_rmi_1_;

    protected IToGUI worksheet_rmi_2_;

    // formula which will be edited if the dialog allows it
    protected CalcFormula edited_formula_;

    /** Model for the JTree */
    // protected TreeModel model;
    /**
     * LK+WN0505 remove: only used for getting DialogIterators; WorksheetDialog
     * knows CalcTree, get Iterator from there
     * 
     * calc_tree_ removed
     */
    // protected CalcTree calc_tree_;
    // protected CalcModelHierarchy calc_hierarchy_;
    /** Used to represent the model. */
    // protected JTree tree_;
    /*
     * Renderer that is used to draw the tree cells
     */
    // protected CustomTreeCellRenderer treeRenderer_;
    /**
     * The editor used to edit the cells in the tree
     */
    // protected CustomTreeCellEditor treeEditor_;
    /** Dialog which is responsible for the communication with the kernel */
    protected IWorksheetDialog dialog_;

    /** Panel which is shows the calc_head_ if forced */
    // protected CalcHeadPanel chp_;
    /**
     * NC060718: The following two fields : active_iterator_ and
     * selected_iterator_ should be deleted/replaced as soon as possible because
     * an CalcIterator represents a direct link to the mathematics engine.
     * Please do not use them in further extensions of the worksheet.
     * 
     * active_iterator_ removed selected_iterator_ removed
     */
    // protected ICalcIterator active_iterator_;
    // protected ICalcIterator selected_iterator_;
    // private CalcModelNode edited_node_;
    // private CalcModelNode selected_node_;
    // protected CalcHead ch_;
    // private JDesktopPane desktop_;
    // private boolean calc_head_visible_;
    // private JInternalFrame panel_frame_;
    /**
     * Create a <code>Worksheet</code> (and display the Specification if
     * requested by the viewStyle according to UserSettings). The given
     * <code>DGuide</code> is connected to the <code>Worksheet</code>.
     * 
     * @param dialogGuide
     *            The dialog guide that is connected to the Worksheet
     * @param desktop
     *            DesktopPane, the Worksheet is added to
     * @see ...{isac.wsdialog.WorksheetDialog#STARTFROM_*}
     */
    public WorksheetStub(IToWorksheetUser windowApplication,
            IWorksheetDialog dialogGuide, JDesktopPane desktop) {
        dialog_ = dialogGuide;
        try {
            // !!!!! works after establishing inheritance !!!!!!!!!!!!!!!!!!!!
            // worksheet_rmi_1_ = new WorksheetRMI(this);//!!!!!!!!!!!!!!!!!!!
            // worksheet_rmi_2_ = new WorksheetRMI(this);//!!!!!!!!!!!!!!!!!!!
            // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

            dialog_.notifyGUIReady();

            dialog_.addDataChangeListener(worksheet_rmi_1_);
            dialog_.registerUIControlListener(worksheet_rmi_2_);
            dialog_.startCalculate();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * 
     * 
     * Description of the method in HTML format, if a link can be given in the
     * running text do this with {@link
     * fully.qualified.Classname#methodName(fully.qualified.Paramclass)}
     * 
     * @param <paramname>
     *            <paramdescription>
     * @return <return value>
     * @exception <exception>
     *                <description when it is thrown>
     * @see <fully.qualified.Classname#methodName(param-classes)>
     * 
     * kept as info for WEB-implementor
     */
    public void startSolving() {
        try {
            // // dialog.newCalculation(ch);
            // System.out.println("in startSolving()");
            // chp_.fillCalcHeadFromTextFields(ch_);
            // if (logger.isInfoEnabled())
            // logger.info("WS->DG: UI_SPECIFY_TO_SOLVE");
            // if (
            dialog_.notifyUserAction(new UserAction(
                    EUIElement.UI_SPECIFY_TO_SOLVE));
            // ) {
            // replaceCalcHeadPanelByWorksheet();
            // } else {
            // JOptionPane.showMessageDialog(this,
            // "Please enter a correct CalcHead",
            // "CalcHead not correct", JOptionPane.WARNING_MESSAGE);
            // }
        } catch (RemoteException e1) {
            logger.error("actionPerformed", e1.detail);
        } catch (DialogProtocolException e2) {
            logger.error("actionPerformed", e2);
            logger.error("e2" + e2);
        }
    }

    /**
     * kept as info for WEB-implementor
     */
    public void compCalcHead() {
        try {
            // chp_.fillCalcHeadFromTextFields(ch_);
            dialog_.notifyUserAction(new UserAction(
                    EUIElement.UI_SPECIFY_COMPLETE_CALCHEAD));
            // chp_.fillTextFieldsFromCalcHead(ch_);
        } catch (Exception e1) {
            logger.error(e1);
        }
    }

    /**
     * kept as info for WEB-implementor
     */
    public void checkCalcHead() {
        try {
            if (logger.isDebugEnabled())
                logger.debug(dialog_);
            dialog_.notifyUserAction(new UserAction(
                    EUIElement.UI_SPECIFY_CHECK_CALCHEAD));
            // chp_.fillTextFieldsFromCalcHead(ch_);
        } catch (Exception e1) {
            logger.error(e1);
        }
    }

    /**
     * Shows CalcHeadPanel. Creates a new InternalFrame on the desktopPane the
     * worksheet is added to and ensures, that this frames stays in front of the
     * Worksheet as long as it is opened
     * 
     * @param calc_head
     *            CalcHead to display in Panel must be implemented according to
     *            GUI.
     */
    private void openCalcHeadPanel(CalcHead calc_head) {

    }

    /**
     * @param action
     * @deprecated in favour of doUIAction(IUIAction action) with introduction
     *             of IToGUI
     */
    public boolean doUIAction(IUserAction action) {
        return false;
    }

    /**
     * add an interactive element on the gui depending on the context
     * 
     * @see isac.interfaces.IToGUI#addUIElement(isac.wsdialog.IUIAction)
     */
    public void addUIElement(IUIAction action) {

    }

    /**
     * remove an interactive element from the gui
     * 
     * @see isac.interfaces.IToGUI#removeUIElement(isac.wsdialog.IUIAction)
     */
    public boolean removeUIElement(IUIAction action) {
        return false;
    }

    /**
     * Services the UI_SOLVE_SHOW_ASSUMPTIONS UIAction. This UIAction is thrown
     * from WorksheetDialog. This method is called only if the returned element
     * is indeed Assumptions and if the element is not null. The data is written
     * in the tacticString field of the edited node.
     * 
     * 
     * @param ass -
     *            the new Assumption
     * 
     */
    private void doShowAssumptions(Assumptions asm) {
    }

    /**
     * Perform the UIAction requested from the WorksheetDialog. The cases depend
     * only on IUIAction; GUI-dependent actions must be caught in the inherited
     * method.
     * 
     * @see isac.interfaces.IToGUI#doUIAction(isac.wsdialog.IUIAction)
     */
    public void doUIAction(IUIAction action) {
        if (logger.isInfoEnabled())
            logger.info("WD->WS doUIAction : action = " + action.getAction()
                    + " Context = " + action.getContext());
        switch (action.getContext()) {
        // TOODO: complete groups of UIElements
        case UI_CONTEXT_CALCULATION:
            break;
        case UI_CONTEXT_CALCELEMENTS:
            switch (action.getAction()) {
            // case UI_SOLVE_TACTIC_APPLIED:
            // if (action instanceof UIActionOnCalcElement) {
            // ICalcElement ice = ((UIActionOnCalcElement) action)
            // .getCalcElement();
            //
            // if (ice == null) {
            // System.out.println("Tactic is null");
            // return;
            // }
            //
            // if (ice instanceof Tactic) {
            // //doTacticApplied((Tactic) ice);------> Swing#doUIAction
            // }
            // else
            // System.out.println("Not my tactic : "
            // + ice.toSMLString() + " is "
            // + ice.getClass().getName());
            // } else {
            // System.out.println("Not my action : "
            // + action.getClass().getName());
            // }
            //
            // break;
            case UI_SOLVE_SHOW_ASSUMPTIONS:
                if (action instanceof UIActionOnCalcElement) {
                    ICalcElement ice = ((UIActionOnCalcElement) action)
                            .getCalcElement();

                    if (ice == null) {
                        System.out.println("Assumption is null");
                        return;
                    }

                    if (ice instanceof Assumptions)
                        doShowAssumptions((Assumptions) ice);
                    else
                        System.out.println("Not my assumption : "
                                + ice.toSMLString() + " is "
                                + ice.getClass().getName());
                }
                break;
            }
            break;
        case UI_CONTEXT_ONEELEMENT:
            switch (action.getAction()) {
            // case UI_SOLVE_SHOW_APPLICABLE_TACTICS:
            // //doShowApplicableTactics();-------------> Swing#doUIAction
            // break;
            case UI_DO_EDIT_FORMULA:
                try {
                    dialog_.notifyUserAction(new UserActionOnCalcElement(
                            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,
                            edited_formula_));
                } catch (RemoteException e) {
                    e.printStackTrace();
                } catch (DialogProtocolException e) {
                    e.printStackTrace();
                }
                break;
            case UI_DO_APPEND_FORMULA:
                try {
                    dialog_.notifyUserAction(new UserActionOnCalcElement(
                            EUIElement.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,
                            edited_formula_));
                } catch (RemoteException e) {
                    e.printStackTrace();
                } catch (DialogProtocolException e) {
                    e.printStackTrace();
                }
                break;
            // case UI_SOLVE_CHANGE_CONTEXT_FORMULA:
            // UIAction ccf_action = (UIAction) action;
            // selected_node_ = calc_hierarchy_.getNodeForValue(ccf_action
            // .getPosition(), calc_hierarchy_.getRoot());
            // //-------------------------------------------> Swing#doUIAction
            // if (!calc_head_visible_) {
            // TreePath sel_path = calc_hierarchy_
            // .getPathToNode(selected_node_);
            // tree_.setSelectionPath(sel_path);
            // tree_.updateUI();
            // //-------------------------------------------> Swing#doUIAction
            // }
            // break;
            default:
                break;
            }
            break;
        // case UI_CONTEXT_SWITCH_PHASES:
        // if (!calc_head_visible_) {
        // CalcHead calc_head = (CalcHead) (((UIActionOnCalcElement) action)
        // .getCalcElement());
        // openCalcHeadPanel(calc_head);
        // } else {
        // //closeCalcHeadPanel();--------------------> Swing#doUIAction
        // }
        // break;
        // case UI_CONTEXT_CALCHEAD:
        // chp_
        // .fillTextFieldsFromCalcHead((CalcHead) (((UIActionOnCalcElement)
        // action)
        // .getCalcElement()));----------------> Swing#doUIAction
        // break;
        }
    }

    /**
     * kept as model for WEB-implementers
     */
    public void calcChanged(CalcChanged event) {
        // ((CalculationModel) model).calcChanged(event);
        // for (int i = 0; i < tree_.getRowCount(); i++) {// WN050428 ???+DG
        // // filters
        // tree_.expandRow(i);
        // }
        // tree_.setEditable(true);
        // try {
        // dialog_.notifyGUIReady();
        // } catch (RemoteException e) {
        // e.printStackTrace();
    }
}
